/****************************************************************************
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *~                             IR Kernel                                    ~
 *~                         Copyright(C) 1993                                ~
 *~                       Software Kinetics Ltd.                             ~
 *~                        All Rights Reserved                               ~
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Copyright Software Kinetics Ltd.
 * 
 * Software Kinetics Ltd. has granted a licence of this software to
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve,
 * reproduce and sell the software.
 *
 * The software is provided to the user "as is" without warranty of
 * any kind.  Neither Software Kinetics Ltd. nor the National Library
 * of Canada warrants,guarantees or makes any representation regarding
 * the use, or the results of the use of the software in terms of
 * accuracy,reliability, or otherwise.  The entire risk as results and
 * performance of the software is assumed by the user. Neither
 * Software Kinetics Ltd. nor the National Library of Canada shall be
 * liable for any direct, indirect, consequential or incidental
 * damages arising out the use of software.
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
 *
 * Program: iraccctr.c
 *
 * Purpose: This file contains the access control service functions.
 *
 * Invocation:
 *
 * Input Arguments:
 *
 * Output/Exit Values: None
 *
 * Functions used:  ira_send_request()
 *
 * Algorithm:
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference: IR KERNEL - TARGET PRELIMINARY SOFTWARE DESIGN DOCUMENT
 *            IR KERNEL - TARGET SOFTWARE DETAIL DESIGN DOCUMENT
 *            IR KERNEL - INTERFACE REQUIREMENTS SPECIFICATION
 *            OSI - ANSI Z39.50 DOCUMENTATION
 *
 * Notes:
 *
 * $Id: iraccctr.c,v 1.1 1993/11/01 17:03:51 irsuprt Exp irsuprt $
 * $Log: iraccctr.c,v $
 * Revision 1.1  1993/11/01  17:03:51  irsuprt
 * Initial revision
 *
 *
 *****************************************************************************/
#include <stdio.h>

#include "ir.h"
#include "irconst.h"
#include "irerror.h"
#include "asn_incl.h"

/*****************************************************************************
 *
 * Function: ir_access conrtol_request
 *
 * Purpose: This function receives a access conrtol request from the Target
 *          Process, validates it, and relays the request to the Origin.
 *
 * Input Arguments:    assocId - pointer to the current association
 *                     PDU_ptr - pointer to the access conrtol request APDU
 *
 * External Variables:
 *
 * Return values:      SUCCESS
 *                     FAILED
 *
 * Output Argument:
 *
 * External Variables:
 *
 * Function used:     ira_send_request()
 *
 * Version:
 *
 * References:
 *
 * Notes:
 *
 ****************************************************************************/

int
ir_access_control_request(assocId, PDU_ptr)
    struct assoc_struct **assocId;
    APDU *PDU_ptr;
{
    int result;

#ifdef DEBUG_LEVEL1
    fprintf(DEBUG_FILE1, "Calling ir_access_control_request ...\n");
#endif

    /* is the access control request APDU valid ? */
    result = SUCCESS;                /* need to be resolved */

    /* send the access control request to the Origin */
    if (result == SUCCESS)
    {
        result = ira_send_request(assocId);
    }

    return (result);

}                                /* End Of Procedure ir_access_control_request */


/*****************************************************************************
 *
 * Function: ir_access_control_response
 *
 * Purpose: This function receives a access control response from the Origin,
 *          validates it, and relays the APDU to the Target Process.
 *
 * Input Arguments:     assocId         - pointer to the current association
 *                      PDU_ptr         - pointer to the APDU
 *
 * External Variables:
 *
 * Return values:     SUCCESS
 *                    INVALID_APDU
 *                    TP_COM_ERROR
 *
 * Output Argument:
 *
 * External Variables:
 *
 * Function called:    ira_service_response()
 *
 * Version:
 *
 * References:
 *
 * Notes:
 *
 ****************************************************************************/

int
ir_access_control_response(assocId, PDU_ptr)
    struct assoc_struct **assocId;
    APDU *PDU_ptr;
{

    int result;

#ifdef DEBUG_LEVEL1
    fprintf(DEBUG_FILE1, "Calling ir_access_control_response ...\n");
#endif

    /* validate the access control response APDU */
    result = SUCCESS;                /* need to be resolved */

    /* service the access control response */
    if (result == SUCCESS)
    {
        result = ira_service_response(assocId);
    }

    return (result);

}                                /* End Of Procedure
                                 * ir_access_control_response */
