/*****************************************************************************
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *~                             IR Kernel                                    ~
 *~                         Copyright(C) 1993                                ~
 *~                       Software Kinetics Ltd.                             ~
 *~                        All Rights Reserved                               ~
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Copyright Software Kinetics Ltd.
 * 
 * Software Kinetics Ltd. has granted a licence of this software to
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve,
 * reproduce and sell the software.
 *
 * The software is provided to the user "as is" without warranty of
 * any kind.  Neither Software Kinetics Ltd. nor the National Library
 * of Canada warrants,guarantees or makes any representation regarding
 * the use, or the results of the use of the software in terms of
 * accuracy,reliability, or otherwise.  The entire risk as results and
 * performance of the software is assumed by the user. Neither
 * Software Kinetics Ltd. nor the National Library of Canada shall be
 * liable for any direct, indirect, consequential or incidental
 * damages arising out the use of software.
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
 *
 * Header:  irstatab.h
 *
 * Purpose: Data definitions for the IR kernel state machine.
 *
 * Index:   section number
 *          section description
 *
 * Version: 93/05/12
 *          Benjamin Yen
 *          reason for change
 *
 * Notes:
 *
 * $Id: irstatab.h,v 1.1 1993/11/01 17:03:51 irsuprt Exp irsuprt $
 * $Log: irstatab.h,v $
 * Revision 1.1  1993/11/01  17:03:51  irsuprt
 * Initial revision
 *
 *
 *****************************************************************************/
/* IR Target State Table Definitions */

/* States   */

#define CLOSED                 1
#define INIT_RECVD             2
#define OPEN                   3
#define SEARCH_RECVD           4
#define PRESENT_RECVD          5
#define DELETE_RECVD           6
#define RESOURCE_REPORT_RECVD  7
#define RESOURCE_CONTROL_SENT  8
#define ACCESS_CONTROL_SENT    9
#define RELEASE_RECVD          10
#define REJECT                 11
#define PROTOCOL_FAILED          0
#define POP_STATE              99

/* Incoming Events    */

#define INIT_REQUEST                1
#define INIT_RESPONSE_ACCEPT        2
#define INIT_RESPONSE_REJECT        3
#define SEARCH_REQUEST              4
#define SEARCH_RESPONSE             5
#define PRESENT_REQUEST             6
#define PRESENT_RESPONSE            7
#define DELETE_REQUEST              8
#define DELETE_RESPONSE             9
#define RESOURCE_REPORT_REQUEST     10
#define RESOURCE_REPORT_RESPONSE    11
#define TRIGGER_RESOURCE_REQUEST    12
#define RESOURCE_REQUEST_RESP       13
#define RESOURCE_REQUEST_NORESP     14
#define RESOURCE_RESPONSE           15
#define ACCESS_CONTROL_REQUEST      16
#define ACCESS_CONTROL_RESPONSE     17

#define A_RELEASE_INDICATION        18
#define IR_RELEASE_RESPONSE         19
#define A_ABORT_INDICATION          20
#define AP_ABORT_INDICATION         21
#define IR_ABORT_REQUEST            22
#define A_ASSOCIATE                 23
#define NON_EVENT                   24

#define ACSE_EVENT                  18 /* boundary between IR and ACSE events */

/* For timeout purpose */
#define INIT_RESPONSE               25
#define TRIGGER_RESOURCE_RESPONSE   26
#define NOT_USED                    27

/* Outgoing Actions    */

#define A_ABORT_REQUEST     21

/* Target State Table Definitions */

/*
 * #define     TRUE        1 #define     FALSE       0
 */

#define     STATES      12
#define     EVENTS      23

/* state table definition */
/* state = 99 means it should get a state from a stack */
extern int stateTable[][];

/* Define the TIMEOUT */
extern int INIT_TIMEOUT_IN_SECONDS;
extern int SEARCH_TIMEOUT_IN_SECONDS;
extern int PRESENT_TIMEOUT_IN_SECONDS;
extern int DELETE_TIMEOUT_IN_SECONDS;
extern int TRIGGER_TIMEOUT_IN_SECONDS;
extern int ACCESS_TIMEOUT_IN_SECONDS;
extern int RESOURCE_TIMEOUT_IN_SECONDS;
extern int IDLE_TIMEOUT_IN_SECONDS;
