/****************************************************************************
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *~                             IR Kernel                                    ~
 *~                         Copyright(C) 1993                                ~
 *~                       Software Kinetics Ltd.                             ~
 *~                        All Rights Reserved                               ~
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Copyright Software Kinetics Ltd.
 * 
 * Software Kinetics Ltd. has granted a licence of this software to
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve,
 * reproduce and sell the software.
 *
 * The software is provided to the user "as is" without warranty of
 * any kind.  Neither Software Kinetics Ltd. nor the National Library
 * of Canada warrants,guarantees or makes any representation regarding
 * the use, or the results of the use of the software in terms of
 * accuracy,reliability, or otherwise.  The entire risk as results and
 * performance of the software is assumed by the user. Neither
 * Software Kinetics Ltd. nor the National Library of Canada shall be
 * liable for any direct, indirect, consequential or incidental
 * damages arising out the use of software.
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
 *
 * Program: irstksta.c
 *
 * Purpose: This file contains the stack functions.
 *
 * Invocation:
 *
 * Input Arguments:
 *
 * Output/Exit Values:
 *
 * Functions used:
 *                  ir_push_state()
 *                  ir_pop_state()
 *                  ir_clear_state_stack()
 *                  ir_view_state()
 *                  ir_find_stack_state()
 *
 * Algorithm:
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference: IR KERNEL - TARGET PRELIMINARY SOFTWARE DESIGN DOCUMENT
 *            IR KERNEL - TARGET SOFTWARE DETAIL DESIGN DOCUMENT
 *            IR KERNEL - INTERFACE REQUIREMENTS SPECIFICATION
 *            OSI - ANSI Z39.50 DOCUMENTATION
 *
 * Notes:
 *
 * $Id: irstksta.c,v 1.1 1993/11/01 17:03:51 irsuprt Exp irsuprt $
 * $Log: irstksta.c,v $
 * Revision 1.1  1993/11/01  17:03:51  irsuprt
 * Initial revision
 *
 *
 *****************************************************************************/
#include <stdio.h>
#include "irconst.h"

struct stack_state
{
    int number;
    struct stack_state *link;
};

/*****************************************************************************
 *
 * Function: ir_push_state
 *
 * Purpose: Create a new stack node with the parameter value and push it
 *          into the stack.
 *
 * Input Arguments: stack_state_head - state stack list pointer
 *                  value - a state
 *
 * Return values: SUCCESS - successfully inserting the new node to the list
 *                FAILED  - failed to insert the new node to the list
 *
 * Output Argument:
 *
 * External Variables:
 *
 * Function used:
 *
 * Algorithm:
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * References:
 *
 * Notes:
 *
 ****************************************************************************/
int
ir_push_state(stack_state_head, value)
    struct stack_state **stack_state_head;
    int value;
{

    struct stack_state *new;

    if ((new = (struct stack_state *) malloc(sizeof(struct stack_state))) == NULL)
        return (FAILED);
    else
    {
        new->number = value;
        new->link = (*stack_state_head);
        (*stack_state_head) = new;
        return (SUCCESS);
    }
}

/*****************************************************************************
 *
 * Function: ir_pop_state
 *
 * Purpose: Pop a service from the stack and free the node.
 *
 * Input Arguments: stack_state_head - state stack list pointer
 *
 * Return values: A state from the top of the stack or zero if it's a empty
 *                stack.
 *
 * Output Argument:
 *
 * External Variables:
 *
 * Function used:
 *
 * Algorithm:
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * References:
 *
 * Notes:
 *
 ****************************************************************************/
int
ir_pop_state(stack_state_head)
    struct stack_state **stack_state_head;
{

    struct stack_state *old;
    int value;

    if ((*stack_state_head))
    {
        value = (*stack_state_head)->number;
        old = (*stack_state_head);
        (*stack_state_head) = (*stack_state_head)->link;
        free(old);
    }
    else
        value = 0;

    return (value);
}


/*****************************************************************************
 *
 * Function: ir_clear_state_stack
 *
 * Purpose: Release all the nodes of the state stack.
 *
 * Input Arguments:
 *                  stack_state_head - the state stack list pointer
 *
 * Return values: None
 *
 * Output Argument:
 *
 * External Variables:
 *
 * Function used: ir_pop_state()
 *
 * Algorithm:
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * References:
 *
 * Notes:
 *
 ****************************************************************************/
void
ir_clear_state_stack(stack_state_head)
    struct stack_state **stack_state_head;
{
    while ((*stack_state_head))
        ir_pop_state(stack_state_head);
}

/*****************************************************************************
 *
 * Function: ir_view_state
 *
 * Purpose: View the state value of all the nodes on the state stack.
 *
 * Input Arguments:
 *                  stack_state_head - the state stack list pointer
 *
 * Return values: None
 *
 * Output Argument:
 *
 * External Variables:
 *
 * Function used:
 *
 * Algorithm:
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * References:
 *
 * Notes:
 *
 ****************************************************************************/
ir_view_state(stack_state_head)
    struct stack_state **stack_state_head;
{
    struct stack_state *ptr;

    ptr = (*stack_state_head);
    printf("state stack =");
    while (ptr)
    {
        printf(" %i", ptr->number);
        ptr = ptr->link;
    }
    printf("\n");
}

/*****************************************************************************
 *
 * Function: ir_find_state_stack
 *
 * Purpose: Verify if the state_value is in the list.
 *
 * Input Arguments:
 *                  stack_state_head - the state stack list pointer
 *                  state_value - the state value to be searched
 *
 * Return values: SUCCESS - found in the list
 *                FAILED  - not found
 *
 * Output Argument:
 *
 * External Variables:
 *
 * Function used:
 *
 * Algorithm:
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * References:
 *
 * Notes:
 *
 ****************************************************************************/
int
ir_find_stack_state(stack_state_head, state_value)
    struct stack_state **stack_state_head;
    int state_value;
{
    struct stack_state *ptr;

    ptr = (*stack_state_head);
    while (ptr)
    {
        if (ptr->number == state_value)
            return (SUCCESS);
        ptr = ptr->link;
    }
    return (FAILED);
}
