/****************************************************************************
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *~                             IR Kernel                                    ~
 *~                         Copyright(C) 1993                                ~
 *~                       Software Kinetics Ltd.                             ~
 *~                        All Rights Reserved                               ~
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Copyright Software Kinetics Ltd.
 * 
 * Software Kinetics Ltd. has granted a licence of this software to
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve,
 * reproduce and sell the software.
 *
 * The software is provided to the user "as is" without warranty of
 * any kind.  Neither Software Kinetics Ltd. nor the National Library
 * of Canada warrants,guarantees or makes any representation regarding
 * the use, or the results of the use of the software in terms of
 * accuracy,reliability, or otherwise.  The entire risk as results and
 * performance of the software is assumed by the user. Neither
 * Software Kinetics Ltd. nor the National Library of Canada shall be
 * liable for any direct, indirect, consequential or incidental
 * damages arising out the use of software.
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
 *
 * Program: irterm.c
 *
 * Purpose: This file contains the termination functions.
 *
 * Invocation:
 *
 * Input Arguments:
 *
 * Output/Exit Values: None
 *
 * Functions used:
 *                  ir_reject()
 *                  ira_abort_request()
 *                  ir_abort_response()
 *                  ir_release_request()
 *                  ir_release_response()
 *
 * Algorithm:
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference: IR KERNEL - TARGET PRELIMINARY SOFTWARE DESIGN DOCUMENT
 *            IR KERNEL - TARGET SOFTWARE DETAIL DESIGN DOCUMENT
 *            IR KERNEL - INTERFACE REQUIREMENTS SPECIFICATION
 *            OSI - ANSI Z39.50 DOCUMENTATION
 *
 * Notes:
 *
 * $Id: irterm.c,v 1.2 1993/11/02 15:41:51 irsuprt Exp irsuprt $
 * $Log: irterm.c,v $
 * Revision 1.2  1993/11/02  15:41:51  irsuprt
 * removed ; of line 99,136,171,208 for ANSI compliance (PROBLEM 13)
 *
 * Revision 1.1  1993/11/01  17:03:51  irsuprt
 * Initial revision
 *
 *
 *****************************************************************************/
#include "irassoc.h"
#include "apdu.h"

/*****************************************************************************
 *
 * Function: ir_reject
 *
 * Purpose: To simulate the reject state in the state machine.
 *
 * Input Arguments:
 *
 * Return values:
 *
 * Output Argument:
 *
 * External Variables:
 *
 * Function used:
 *
 * Algorithm:
 *
 * Version:
 *
 * References:
 *
 * Notes:
 *
 ****************************************************************************/
ir_reject(struct assoc_struct ** assocId, APDU * PDU_ptr)
{

#ifdef DEBUG_LEVEL1
    fprintf(DEBUG_FILE1, "*** ir_reject_request\n");
#endif
}

/*****************************************************************************
 *
 * Function: ira_abort_request
 *
 * Purpose: To simulate OSI IR-ABORT request. It will just terminate for now.
 *          To call tp_terminate_service() using RPC call.
 *
 * Input Arguments:
 *
 * Return values:
 *
 * Output Argument:
 *
 * External Variables:
 *
 * Function used:
 *
 * Algorithm:
 *
 * Version:
 *
 * References:
 *
 * Notes:
 *
 ****************************************************************************/
ira_abort_request(struct assoc_struct ** assocId)
{

#ifdef DEBUG_LEVEL1
    fprintf(DEBUG_FILE1, "*** ira_abort_request\n");
#endif

    /* Send the IR-ABORT request PDU to target process */
    /* To call tp_terminate_service() using RPC call. */
}

/*****************************************************************************
 *
 * Function: ir_abort_response
 *
 * Purpose: To simulate OSI IR-ABORT response. It will just terminate for now.
 *
 * Input Arguments:
 *
 * Return values:
 *
 * Output Argument:
 *
 * External Variables:
 *
 * Function used:
 *
 * Algorithm:
 *
 * Version:
 *
 * References:
 *
 * Notes:
 *
 ****************************************************************************/
ir_abort_response(struct assoc_struct ** assocId, APDU * PDU_ptr)
{

#ifdef DEBUG_LEVEL1
    fprintf(DEBUG_FILE1, "*** ir_abort_response\n");
#endif

    /* Send the IR-ABORT response PDU back to origin system */
}

/*****************************************************************************
 *
 * Function: ir_release_request
 *
 * Purpose: To simulate OSI IR-ABORT request. It will just terminate for now.
 *          To call tp_terminate_service() using RPC call.
 *
 * Input Arguments:
 *
 * Return values:
 *
 * Output Argument:
 *
 * External Variables:
 *
 * Function used:
 *
 * Algorithm:
 *
 * Version:
 *
 * References:
 *
 * Notes:
 *
 ****************************************************************************/
ir_release_request(struct assoc_struct ** assocId)
{

#ifdef DEBUG_LEVEL1
    fprintf(DEBUG_FILE1, "*** ir_release_request\n");
#endif

    /* Send the IR-RELEASE request PDU to target process */
    /* To call tp_terminate_service() using RPC call. */
}

/*****************************************************************************
 *
 * Function: ir_release_response
 *
 * Purpose: To simulate OSI IR-ABORT response. It will just terminate for now.
 *
 * Input Arguments:
 *
 * External Variables:
 *
 * Output Return values:
 *
 * Output Argument:
 *
 * External Variables:
 *
 * Function used:
 *
 * Algorithm:
 *
 * Version:
 *
 * References:
 *
 * Notes:
 *
 ****************************************************************************/
ir_release_response(struct assoc_struct ** assocId)
{

#ifdef DEBUG_LEVEL1
    fprintf(DEBUG_FILE1, "*** ir_release_response\n");
#endif

    /* Send the IR-RELEASE response PDU back to origin system */
}

/*****************************************************************************
 *
 * Function: ira_terminate
 *
 * Purpose: To inform the target process to abort and clean up.
 *
 * Input Arguments:
 *
 * External Variables:
 *
 * Output Return values:
 *
 * Output Argument:
 *
 * External Variables:
 *
 * Function used:
 *
 * Algorithm:
 *
 * Version:
 *
 * References:
 *
 * Notes:
 *
 ****************************************************************************/
ira_terminate(struct assoc_struct ** assocId)
{

#ifdef DEBUG_LEVEL1
    fprintf(DEBUG_FILE1, "*** ira_terminate\n");
#endif

    /* Send the special termination signal to target process to abort and */
    /* clean up.                                                          */
}
