/****************************************************************************
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *~                             IR Kernel                                    ~
 *~                         Copyright(C) 1993                                ~
 *~                       Software Kinetics Ltd.                             ~
 *~                        All Rights Reserved                               ~
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Copyright Software Kinetics Ltd.
 * 
 * Software Kinetics Ltd. has granted a licence of this software to
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve,
 * reproduce and sell the software.
 *
 * The software is provided to the user "as is" without warranty of
 * any kind.  Neither Software Kinetics Ltd. nor the National Library
 * of Canada warrants,guarantees or makes any representation regarding
 * the use, or the results of the use of the software in terms of
 * accuracy,reliability, or otherwise.  The entire risk as results and
 * performance of the software is assumed by the user. Neither
 * Software Kinetics Ltd. nor the National Library of Canada shall be
 * liable for any direct, indirect, consequential or incidental
 * damages arising out the use of software.
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
 *
 * Program: tpdelete.c
 *
 * Purpose: This file contains the target process black box delete result set
 *          functions.
 *
 * Invocation:
 *
 * Input Arguments:
 *
 * Output/Exit Values: None
 *
 * Functions used:
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference: IR KERNEL - TARGET PRELIMINARY SOFTWARE DESIGN DOCUMENT
 *            IR KERNEL - TARGET SOFTWARE DETAIL DESIGN DOCUMENT
 *            IR KERNEL - INTERFACE REQUIREMENTS SPECIFICATION
 *            OSI - ANSI Z39.50 DOCUMENTATION
 *
 * Notes:
 *
 * $Id: tpdelete.c,v 1.1 1993/11/01 17:03:51 irsuprt Exp irsuprt $
 * $Log: tpdelete.c,v $
 * Revision 1.1  1993/11/01  17:03:51  irsuprt
 * Initial revision
 *
 *
 *****************************************************************************/

#include <stdio.h>
#include <rpc/rpc.h>
#include <fcntl.h>
#include <pwd.h>
#include <ctype.h>
#include <sys/errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>
#include "tp.h"
#include "asn_incl.h"
#include "irerror.h"
#include "tpconst.h"
#include "tpbbox.h"

AsnInt DELETE_REFERENCE_ID_LENGTH;
char *DELETE_REFERENCE_ID;

/**************************** Extern Global Ids **************************/

extern int errno;

/*****************************************************************************
 *
 * Function: tp_fill_deleteResultSet_response
 *
 * Purpose: This function fills and creates the delete response PDU.
 *
 * Input Arguments:
 *              delete_result - delete result code to response
 *
 * Return values: SUCCESS or FAILED
 *
 * Output Arguments:
 *              PDU_ptr - delete response PDU pointer
 *
 * External Variables:
 *
 * Function used:
 *
 * Algorithm:
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference:
 *
 * Notes:
 *
 ****************************************************************************/
tp_fill_deleteResultSet_response(delete_result, PDU_ptr)
    int delete_result;
    APDU *PDU_ptr;
{

    NamePlusRecord *data_ptr;

#ifdef DEBUG_LEVEL1
    fprintf(DEBUG_FILE1, "BBOX: calling tp_fill_deleteResultSet_response\n");
#endif

    PDU_ptr->a.deleteResultSetResp = (struct DeleteResultSetResp *) malloc(sizeof(struct DeleteResultSetResp));

    PDU_ptr->choiceId = APDU_DELETERESULTSETRESP;

    PDU_ptr->a.deleteResultSetResp->referenceId.octetLen = DELETE_REFERENCE_ID_LENGTH;
    PDU_ptr->a.deleteResultSetResp->referenceId.octs = (char *) malloc(DELETE_REFERENCE_ID_LENGTH);
    memcpy(PDU_ptr->a.deleteResultSetResp->referenceId.octs, DELETE_REFERENCE_ID, DELETE_REFERENCE_ID_LENGTH);

    PDU_ptr->a.deleteResultSetResp->deleteOperStatus = delete_result;

    PDU_ptr->a.deleteResultSetResp->deleteListStatuses = (AsnList *) malloc(sizeof(AsnList));
    PDU_ptr->a.deleteResultSetResp->deleteListStatuses->first = NULL;
    PDU_ptr->a.deleteResultSetResp->deleteListStatuses->last = NULL;
    PDU_ptr->a.deleteResultSetResp->deleteListStatuses->curr = NULL;
    PDU_ptr->a.deleteResultSetResp->deleteListStatuses->count = 0;
    PDU_ptr->a.deleteResultSetResp->deleteListStatuses->dataSize = 0;

    PDU_ptr->a.deleteResultSetResp->numberNotDeleted = (AsnInt *) malloc(sizeof(AsnInt));
    *PDU_ptr->a.deleteResultSetResp->numberNotDeleted = 0;

    PDU_ptr->a.deleteResultSetResp->bulkStatuses = (AsnList *) malloc(sizeof(AsnList));
    PDU_ptr->a.deleteResultSetResp->bulkStatuses->first = NULL;
    PDU_ptr->a.deleteResultSetResp->bulkStatuses->last = NULL;
    PDU_ptr->a.deleteResultSetResp->bulkStatuses->curr = NULL;
    PDU_ptr->a.deleteResultSetResp->bulkStatuses->count = 0;
    PDU_ptr->a.deleteResultSetResp->bulkStatuses->dataSize = 0;

    if (delete_result == 0)
    {
        PDU_ptr->a.deleteResultSetResp->deleteMessage.octetLen = sizeof("DELETE SUCCESSFUL") - 1;
        PDU_ptr->a.deleteResultSetResp->deleteMessage.octs = (char *) malloc(PDU_ptr->a.deleteResultSetResp->deleteMessage.octetLen);
        memcpy(PDU_ptr->a.deleteResultSetResp->deleteMessage.octs, "DELETE SUCCESSFUL", PDU_ptr->a.deleteResultSetResp->deleteMessage.octetLen);
    }
    else if (delete_result == 4)
    {
        PDU_ptr->a.deleteResultSetResp->deleteMessage.octetLen = sizeof("ACCESS CONTROL FAILED") - 1;
        PDU_ptr->a.deleteResultSetResp->deleteMessage.octs = (char *) malloc(PDU_ptr->a.deleteResultSetResp->deleteMessage.octetLen);
        memcpy(PDU_ptr->a.deleteResultSetResp->deleteMessage.octs, "ACCESS CONTROL FAILED", PDU_ptr->a.deleteResultSetResp->deleteMessage.octetLen);
    }
    else if (delete_result == 5)
    {
        PDU_ptr->a.deleteResultSetResp->deleteMessage.octetLen = sizeof("RESOURCE CONTROL FAILED") - 1;
        PDU_ptr->a.deleteResultSetResp->deleteMessage.octs = (char *) malloc(PDU_ptr->a.deleteResultSetResp->deleteMessage.octetLen);
        memcpy(PDU_ptr->a.deleteResultSetResp->deleteMessage.octs, "RESOURCE CONTROL FAILED", PDU_ptr->a.deleteResultSetResp->deleteMessage.octetLen);
    }
    else
    {
        PDU_ptr->a.deleteResultSetResp->deleteMessage.octetLen = sizeof("MANDATORY FIELDS ARE INCOMPLETE") - 1;
        PDU_ptr->a.deleteResultSetResp->deleteMessage.octs = (char *) malloc(PDU_ptr->a.deleteResultSetResp->deleteMessage.octetLen);
        memcpy(PDU_ptr->a.deleteResultSetResp->deleteMessage.octs, "MANDATORY FIELDS ARE INCOMPLETE", PDU_ptr->a.deleteResultSetResp->deleteMessage.octetLen);
    }

    return (SUCCESS);
}

/*****************************************************************************
 *
 * Function: tp_create_deleteResultSet_response
 *
 * Purpose: This function creates the delete response PDU, encodes, and
 *          creates the delete response APDU for sending to IR kernel.
 *
 * Input Arguments:
 *              delete_result - delete result code to response
 *              data_ptr - temporary data buffer for encoding process
 *
 * Return values: SUCCESS or FAILED
 *
 * Output Arguments:
 *              outputBuf - encoded APDU pointer
 *              APDU_length - the size of the encoded APDU
 *
 * External Variables:
 *
 * Function used:
 *              tp_fill_deleteResultSet_response()
 *              PrintDeleteResultSetResp()
 *              InitNibbleMem()
 *              SBufInstallData()
 *              SBufResetInWriteRvsMode()
 *              BEncAPDU()
 *              ir_fwrite()
 *              FreeDeleteResultSetResp()
 *
 * Algorithm:
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference:
 *
 * Notes:
 *
 ****************************************************************************/
tp_create_deleteResultSet_response(delete_result, outputBuf, data_ptr, APDU_length)
    int delete_result;
    SBuf *outputBuf;
    char **data_ptr;
    int *APDU_length;
{
    APDU PDU_ptr;

#ifdef DEBUG_LEVEL1
    fprintf(DEBUG_FILE1, "BBOX:Calling tp_create_deleteResultSet_response()\n");
#endif

    tp_fill_deleteResultSet_response(delete_result, &PDU_ptr);

#ifdef DEBUG_LEVEL2
    fprintf(DEBUG_FILE2, "BBOX: tp_create_deleteResultSet_response, begin dump data:\n");
    PrintDeleteResultSetResp(DEBUG_FILE2, PDU_ptr.a.deleteResultSetResp, 0);
    fprintf(DEBUG_FILE2, "BBOX: tp_create_deleteResultSet_response, end dump data:\n");
#endif

    InitNibbleMem(MAXMESSAGESIZE, MAXMESSAGESIZE);

    SBufInstallData(outputBuf, (*data_ptr), MAXMESSAGESIZE * 2);
    SBufResetInWriteRvsMode(outputBuf);

    *APDU_length = BEncAPDU(outputBuf, &PDU_ptr);

#ifdef DEBUG_LEVEL2
    fprintf(DEBUG_FILE2, "BBOX:delete response APDU Dump from tpdelete\n");
    ir_fwrite(SBufDataPtr(outputBuf), SBufDataLen(outputBuf), 1, DEBUG_FILE2);
#endif

    FreeDeleteResultSetResp(PDU_ptr.a.deleteResultSetResp);

    return (SUCCESS);
}

/*****************************************************************************
 *
 * Function: tp_check_delete_request
 *
 * Purpose: This function checks the delete request PDU to make sure the
 *          mandatory fields are present and valid.
 *
 * Input Arguments: PDU - delete request PDU pointer
 *
 * Return values: SUCCESS or FAILED
 *
 * Output Arguments:
 *
 * External Variables:
 *
 * Function used:
 *              display_error()
 *
 * Algorithm:
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference:
 *
 * Notes:
 *
 ****************************************************************************/
tp_check_delete_request(PDU_ptr)
    APDU *PDU_ptr;
{

    /* save reference ID */
    DELETE_REFERENCE_ID_LENGTH = PDU_ptr->a.deleteResultSetReq->referenceId.octetLen;
    DELETE_REFERENCE_ID = (char *) malloc(PDU_ptr->a.deleteResultSetReq->referenceId.octetLen);
    memcpy(DELETE_REFERENCE_ID, PDU_ptr->a.deleteResultSetReq->referenceId.octs, PDU_ptr->a.deleteResultSetReq->referenceId.octetLen);

    if (!(PDU_ptr->a.deleteResultSetReq->deleteOperation == 0 ||
          PDU_ptr->a.deleteResultSetReq->deleteOperation == 1))
    {
        display_error("BBOX", ERR_INVALID_DELETE_OPERATION, -1);
        return (FAILED);
    }

    return (SUCCESS);
}

/*****************************************************************************
 *
 * Function: tp_process_deleteResultSet_request
 *
 * Purpose: This function performs the following functions:
 *              - checks the delete request PDU,
 *              - performs the security checks,
 *              - performs the resource verification, and
 *              - creates and sends the delete response APDU to IR kernel.
 *
 * Input Arguments:
 *              PDU - delete request PDU pointer
 *              data_ptr - temporary data pointer for ASN functions
 *              socketFd - the socket Fd for communicating to IR kernel
 *
 * Return values: SUCCESS or FAILED
 *
 * Output Arguments:
 *
 * External Variables:
 *
 * Function used:
 *              tp_check_delete_request()
 *              FreeDeleteResultSetReq()
 *              tp_access_control()
 *              tp_resource_control()
 *              tp_create_deleteResultSet_response()
 *              tpa_send_APDU()
 *              display_error()
 *
 * Algorithm:
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference:
 *
 * Notes:
 *
 ****************************************************************************/
tp_process_deleteResultSet_request(PDU, data_ptr, socketFd)
    APDU *PDU;
    char **data_ptr;
    int socketFd;
{
    int result;
    long APDU_length;
    SBuf APDU_ptr;

#ifdef DEBUG_LEVEL1
    fprintf(DEBUG_FILE1, "BBOX: calling tp_process_deleteResultSet_request\n");
#endif

    result = tp_check_delete_request(PDU);
    FreeDeleteResultSetReq(PDU->a.deleteResultSetReq);

    if (result == SUCCESS)
    {
        if (tp_access_control(socketFd, data_ptr, DELETE_REFERENCE_ID_LENGTH, DELETE_REFERENCE_ID) == SUCCESS)
        {
            if (tp_resource_control(socketFd, data_ptr, DELETE_REFERENCE_ID_LENGTH, DELETE_REFERENCE_ID) == FAILED)
            {
                display_error("BBOX", ERR_RESOURCE_CONTROL_FAILED, -1);
                /*
                 * create a delete response with result 'resource control
                 * failed'
                 */
                tp_create_deleteResultSet_response(5, &APDU_ptr, data_ptr, &APDU_length);
            }
            else
            {
                /* create a good delete response */
                tp_create_deleteResultSet_response(0, &APDU_ptr, data_ptr, &APDU_length);
            }
        }
        else
        {
            display_error("BBOX", ERR_ACCESS_CONTROL_FAILED, -1);
            /* create a delete response with result 'access control failed' */
            tp_create_deleteResultSet_response(4, &APDU_ptr, data_ptr, &APDU_length);
        }
    }
    else
    {
        /*
         * create a delete response with result 'mandatory fields are
         * incomplete'
         */
        tp_create_deleteResultSet_response(2, &APDU_ptr, data_ptr, &APDU_length);
    }

    if (tpa_send_APDU(socketFd, &APDU_ptr, (int) APDU_length) <= 0)
    {
        display_error("TP_BBOX", ERR_SEND_DATA_TO_IR_FROM_TP, errno);
        return (FAILED);
    }

#ifdef DEBUG_LEVEL1
    else
        fprintf(DEBUG_FILE1, "BBOX: tp_process_deleteResultSet_request: %ld deleteResultSet response sent\n", APDU_length);
#endif

    return (SUCCESS);
}
