/****************************************************************************
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *~                             IR Kernel                                    ~
 *~                         Copyright(C) 1993                                ~
 *~                       Software Kinetics Ltd.                             ~
 *~                        All Rights Reserved                               ~
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Copyright Software Kinetics Ltd.
 * 
 * Software Kinetics Ltd. has granted a licence of this software to
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve,
 * reproduce and sell the software.
 *
 * The software is provided to the user "as is" without warranty of
 * any kind.  Neither Software Kinetics Ltd. nor the National Library
 * of Canada warrants,guarantees or makes any representation regarding
 * the use, or the results of the use of the software in terms of
 * accuracy,reliability, or otherwise.  The entire risk as results and
 * performance of the software is assumed by the user. Neither
 * Software Kinetics Ltd. nor the National Library of Canada shall be
 * liable for any direct, indirect, consequential or incidental
 * damages arising out the use of software.
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
 *
 * Program: tpinit.c
 *
 * Purpose: This is the target process black box init program.
 *
 * Invocation:
 *
 * Input Arguments:
 *
 * Output/Exit Values: None
 *
 * Functions used:
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference: IR KERNEL - TARGET PRELIMINARY SOFTWARE DESIGN DOCUMENT
 *            IR KERNEL - TARGET SOFTWARE DETAIL DESIGN DOCUMENT
 *            IR KERNEL - INTERFACE REQUIREMENTS SPECIFICATION
 *            OSI - ANSI Z39.50 DOCUMENTATION
 *
 * Notes:
 *
 * $Id: tpinit.c,v 1.2 1993/11/02 15:43:50 irsuprt Exp irsuprt $
 * $Log: tpinit.c,v $
 * Revision 1.2  1993/11/02  15:43:50  irsuprt
 * changed #ifdef 0 to #ifdef ACTIVE_TEMPORARY of line 446 for ANSI compliace
 * (PROBLEM 13)
 *
 * Revision 1.1  1993/11/01  17:03:51  irsuprt
 * Initial revision
 *
 *
 *****************************************************************************/

#include <stdio.h>
#include <rpc/rpc.h>
#include <fcntl.h>
#include <pwd.h>
#include <ctype.h>
#include <sys/errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>
#include "tp.h"
#include "asn_incl.h"
#include "irerror.h"
#include "tpconst.h"
#include "tpbbox.h"


/**************************** Global Ids *********************************/

char IMPLEMENTATION_ID[] = "IR Z39.50 Kernel V2.0, FR 1993";
char IMPLEMENTATION_NAME[] = "National Library IR Z39.50 Kernel";
char IMPLEMENTATION_VERSION[] = "2.0";

char INIT_REQUEST_OPTIONS[7] = "1111111";

AsnInt INIT_REFERENCE_ID_LENGTH;
char *INIT_REFERENCE_ID;

/**************************** Extern Global Ids **************************/

extern int errno;

/*****************************************************************************
 *
 * Function: tp_fill_init_response
 *
 * Purpose: This function fills and creates the init response PDU.
 *
 * Input Arguments:
 *              accept - 1 (init response filled with Accept)
 *                       0 (init response filled with reject)
 *              protocol_version - replied protocol version
 *              options - replied options
 *
 * Return values: SUCCESS or FAILED
 *
 * Output Arguments:
 *              PDU_ptr - init response PDU pointer
 *
 * External Variables:
 *
 * Function used:
 *
 * Algorithm:
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference:
 *
 * Notes:
 *
 ****************************************************************************/
tp_fill_init_response(PDU_ptr, accept, protocol_version, options)
    APDU *PDU_ptr;
    int accept;
    char protocol_version;
    char options;
{

#ifdef DEBUG_LEVEL1
    fprintf(DEBUG_FILE1, "BBOX: calling tp_fill_init_response\n");
#endif

    PDU_ptr->a.initResp = (struct InitializeResp *) malloc(sizeof(struct InitializeResp));
    memset(PDU_ptr->a.initResp, NULL, sizeof(struct InitializeResp));

    PDU_ptr->choiceId = APDU_INITRESP;

    PDU_ptr->a.initResp->referenceId.octetLen = INIT_REFERENCE_ID_LENGTH;
    PDU_ptr->a.initResp->referenceId.octs = (char *) malloc(INIT_REFERENCE_ID_LENGTH);
    memcpy(PDU_ptr->a.initResp->referenceId.octs, INIT_REFERENCE_ID, INIT_REFERENCE_ID_LENGTH);

    PDU_ptr->a.initResp->protocolVersion.bitLen = 1;
    PDU_ptr->a.initResp->protocolVersion.bits = malloc(1);
    PDU_ptr->a.initResp->protocolVersion.bits[0] = protocol_version;

    PDU_ptr->a.initResp->options.bitLen = 1;
    PDU_ptr->a.initResp->options.bits = malloc(1);
    /* support 0-search, 1-present, 2-delete, 3-resource report, */
    /* 4-trig resc ctrl, 5-resc ctrl, 6-access control. */
    PDU_ptr->a.initResp->options.bits[0] = options;

    PDU_ptr->a.initResp->preferredMessageSize = MAXMESSAGESIZE;
    PDU_ptr->a.initResp->maximumRecordSize = MAXMESSAGESIZE;
    if (accept)
        PDU_ptr->a.initResp->result = TRUE;        /* accept */
    else
        PDU_ptr->a.initResp->result = FALSE;        /* reject */

    PDU_ptr->a.initResp->implementationId.octetLen = sizeof(IMPLEMENTATION_ID) - 1;
    PDU_ptr->a.initResp->implementationId.octs = (char *) malloc(PDU_ptr->a.initResp->implementationId.octetLen);
    memcpy(PDU_ptr->a.initResp->implementationId.octs, IMPLEMENTATION_ID, PDU_ptr->a.initResp->implementationId.octetLen);

    PDU_ptr->a.initResp->implementationName.octetLen = sizeof(IMPLEMENTATION_NAME) - 1;
    PDU_ptr->a.initResp->implementationName.octs = (char *) malloc(PDU_ptr->a.initResp->implementationName.octetLen);
    memcpy(PDU_ptr->a.initResp->implementationName.octs, IMPLEMENTATION_NAME, PDU_ptr->a.initResp->implementationName.octetLen);

    PDU_ptr->a.initResp->implementationVersion.octetLen = sizeof(IMPLEMENTATION_VERSION) - 1;
    PDU_ptr->a.initResp->implementationVersion.octs = (char *) malloc(PDU_ptr->a.initResp->implementationVersion.octetLen);
    memcpy(PDU_ptr->a.initResp->implementationVersion.octs, IMPLEMENTATION_VERSION, PDU_ptr->a.initResp->implementationVersion.octetLen);

    return (SUCCESS);

}

/*****************************************************************************
 *
 * Function: tp_create_init_response
 *
 * Purpose: This function creates the init response PDU, encodes, and
 *          creates the init response APDU for sending to IR kernel.
 *
 * Input Arguments:
 *              accept - 1 (init response filled with Accept)
 *                       0 (init response filled with reject)
 *              protocol_version - replied protocol version
 *              options - replied options
 *              data_ptr - temporary data buffer for encoding process
 *
 * Return values: SUCCESS or FAILED
 *
 * Output Arguments:
 *              outputBuf - encoded APDU pointer
 *              APDU_length - the size of the encoded APDU
 *
 * External Variables:
 *
 * Function used:
 *              tp_fill_init_response()
 *              InitNibbleMem()
 *              SBufInstallData()
 *              SBufResetInWriteRvsMode()
 *              BEncAPDU()
 *              ir_fwrite()
 *              FreeInitializeResp()
 *
 * Algorithm:
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference:
 *
 * Notes:
 *
 ****************************************************************************/
tp_create_init_response(accept, outputBuf, protocol_version, options, data_ptr, APDU_length)
    int accept;                        /* boolean */
    SBuf *outputBuf;
    char protocol_version;
    char options;
    char **data_ptr;
    int *APDU_length;
{
    APDU PDU_ptr;

#ifdef DEBUG_LEVEL1
    fprintf(DEBUG_FILE1, "BBOX:Calling tp_create_init_response()\n");
#endif

    tp_fill_init_response(&PDU_ptr, accept, protocol_version, options);

    InitNibbleMem(MAXMESSAGESIZE, MAXMESSAGESIZE);

    SBufInstallData(outputBuf, (*data_ptr), MAXMESSAGESIZE * 2);
    SBufResetInWriteRvsMode(outputBuf);

    *APDU_length = BEncAPDU(outputBuf, &PDU_ptr);

#ifdef DEBUG_LEVEL2
    fprintf(DEBUG_FILE2, "BBOX:Init Response APDU Dump from tpinit\n");
    ir_fwrite(SBufDataPtr(outputBuf), SBufDataLen(outputBuf), 1, DEBUG_FILE2);
#endif

    FreeInitializeResp(PDU_ptr.a.initResp);

    return (SUCCESS);
}

/*****************************************************************************
 *
 * Function: tp_check_init_request
 *
 * Purpose: This function checks the init request PDU to make sure the
 *          mandatory fields are present and valid.
 *
 * Input Arguments: PDU_ptr - init request PDU pointer
 *
 * Return values: SUCCESS or FAILED
 *
 * Output Arguments:
 *              protocol_version - final protocol version
 *              options - final options
 *              check_password_already - indicating if the password has been
 *                                       verified.
 *
 * External Variables:
 *
 * Function used:
 *              display_error()
 *
 * Algorithm:
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference:
 *
 * Notes:
 *
 ****************************************************************************/
tp_check_init_request(PDU_ptr, protocol_version, options, check_password_already)
    APDU *PDU_ptr;
    char *protocol_version;
    char *options;
    int *check_password_already;
{

    int i;

#ifdef DEBUG_LEVEL1
    fprintf(DEBUG_FILE1, "BBOX: calling tp_check_init_request\n");
#endif

/* Initialize the Ids */
*check_password_already = FALSE;

    /* save reference ID */
    INIT_REFERENCE_ID_LENGTH = PDU_ptr->a.initReq->referenceId.octetLen;
    INIT_REFERENCE_ID = (char *) malloc(PDU_ptr->a.initReq->referenceId.octetLen);
    memcpy(INIT_REFERENCE_ID, PDU_ptr->a.initReq->referenceId.octs, PDU_ptr->a.initReq->referenceId.octetLen);

    /* checking the mandatory fields */

#ifdef DEBUG_LEVEL3
    fprintf(DEBUG_FILE3, "tp_check_init_request: protocolVersion.bitLen=%d\n", PDU_ptr->a.initReq->protocolVersion.bitLen);
    fprintf(DEBUG_FILE3, "tp_check_init_request: protocolVersion.bits[0]=%x\n", PDU_ptr->a.initReq->protocolVersion.bits[0] & 0xff);
#endif

    if ((PDU_ptr->a.initReq->protocolVersion.bitLen <= 0) ||
            (PDU_ptr->a.initReq->protocolVersion.bits[0] & 0xff <= 0)
        )
    {
        display_error("BBOX", ERR_EMPTY_INIT_PROTOCOL_VERSION, -1);
        return (FAILED);
    }

    if (PDU_ptr->a.initReq->preferredMessageSize <= 0)
    {
        display_error("BBOX", ERR_EMPTY_INIT_PREFERRED_MESSAGE_SIZE, -1);
    }

    if (PDU_ptr->a.initReq->maximumRecordSize <= 0)
    {
        display_error("BBOX", ERR_EMPTY_INIT_MAXIMUM_RECORD_SIZE, -1);
    }

    /* end of checking the mandatory fields */

    /*
     * support 0-search 80H, 1-present 40H, 2-delete 20H, 3-resource report
     * 10H, 4-trig resc ctrl 08H, 5-resc ctrl 04H, 6-access control 02H. The
     * target process will downgrade to SR if the client is a SR client ie.
     * access control and resource control are "will support" not "will use".
     * In another words, the access control and resource control are not
     * enforced.
     */

#ifdef DEBUG_LEVEL3
    fprintf(DEBUG_FILE3, "BBOX: INIT REQUEST OPTIONS from client: %x\n", PDU_ptr->a.initReq->options.bits[0]);
    fprintf(DEBUG_FILE3, "BBOX: INIT REQUEST PROTOCOL VERSION from client =%x\n", PDU_ptr->a.initReq->protocolVersion.bits[0]);
#endif

#ifdef ENFORCE_ACCESS_RESOURCE_CONTROL

#ifdef DEBUG_LEVEL1
	fprintf(DEBUG_FILE1, "ACCESS/RESOURCE CONTROL IS ENFORCED\n");
#endif

    /* if IR protocol bit is not set */
    if ((PDU_ptr->a.initReq->protocolVersion.bits[0] & IR_PROTOCOL_BIT) == 0)
    {
        display_error("BBOX", ERR_INVALID_INIT_PROTOCOL_VERSION, -1);
        return (FAILED);
    }

    /* if access control and resource control option bits are not set */
    if (((PDU_ptr->a.initReq->options.bits[0] & ACCESS_CONTROL_BIT) == 0) || ((PDU_ptr->a.initReq->options.bits[0] & RESOURCE_CONTROL_BIT) == 0))
    {
        display_error("BBOX", ERR_INVALID_INIT_REQUEST_OPTIONS, -1);
        return (FAILED);
    }

    *protocol_version = 0xc0;        /* SR+IR */
    *options = 0xfe;                /* all on */
    ACCESS_CONTROL_REQUIRED = TRUE;
    RESOURCE_CONTROL_REQUIRED = TRUE;

    if (PDU_ptr->a.initReq->idAuthentication.octetLen > 0)
	{
#ifdef DEBUG_LEVEL1
		fprintf(DEBUG_FILE1, "BBOX: tp_check_init_request: idAuthentification has password\n");
#endif
        if (tp_valid_user(PDU_ptr, CHECK_INIT_PASSWORD))
		{
#ifdef DEBUG_LEVEL1
			fprintf(DEBUG_FILE1, "BBOX: tp_check_init_request: ACCESS CONTROL PASSWORD IS OKAY\n");
#endif
            *check_password_already = TRUE;
		}
		else
		{
#ifdef DEBUG_LEVEL1
			fprintf(DEBUG_FILE1, "BBOX: tp_check_init_request: ACCESS CONTROL PASSWORD IS BAD\n");
#endif
		}
#ifdef DEBUG_LEVEL1
		fprintf(DEBUG_FILE1, "BBOX: tp_check_init_request: idAuthentification has no password\n");
#endif
	}

#else

#ifdef DEBUG_LEVEL1
	fprintf(DEBUG_FILE1, "BBOX: ACCESS/RESOURCE CONTROL IS NOT ENFORCED\n");
#endif

    /*
     * if the client is a SR system and the access and resource control are
     * not enforced, downgrade to SR.
     */

    if ((((PDU_ptr->a.initReq->protocolVersion.bits[0] & SR_PROTOCOL_BIT) == SR_PROTOCOL_BIT) && ((PDU_ptr->a.initReq->protocolVersion.bits[0] & IR_PROTOCOL_BIT) == 0)) ||
            (((PDU_ptr->a.initReq->options.bits[0] & ACCESS_CONTROL_BIT) == 0) || ((PDU_ptr->a.initReq->options.bits[0] & RESOURCE_CONTROL_BIT) == 0))
        )
    {
        /* downgrade to SR */
        *protocol_version = PDU_ptr->a.initReq->protocolVersion.bits[0];
        *options = PDU_ptr->a.initReq->options.bits[0];
        ACCESS_CONTROL_REQUIRED = FALSE;
        RESOURCE_CONTROL_REQUIRED = FALSE;

#ifdef DEBUG_LEVEL1
        fprintf(DEBUG_FILE1, "BBOX: check_init_req: down grade to SR\n");
#endif

    }
    else if ((PDU_ptr->a.initReq->protocolVersion.bits[0] & IR_PROTOCOL_BIT) == IR_PROTOCOL_BIT)
    {
        /* if IR */
        *protocol_version = 0xc0;        /* SR+IR */
        *options = 0xfe;        /* all on */
        ACCESS_CONTROL_REQUIRED = TRUE;
        RESOURCE_CONTROL_REQUIRED = TRUE;

        *check_password_already = FALSE;
        if (PDU_ptr->a.initReq->idAuthentication.octetLen > 0)
            if (tp_valid_user(PDU_ptr, CHECK_INIT_PASSWORD))
                *check_password_already = TRUE;

#ifdef DEBUG_LEVEL1
        fprintf(DEBUG_FILE1, "BBOX: check_init_req: IR protocol\n");
#endif

    }
    else
    {
        display_error("BBOX", ERR_INVALID_INIT_PROTOCOL_VERSION, -1);
        return (FAILED);
    }
#endif /* end of #ifdef ENFORCE_ACCESS_RESOURCE_CONTROL */

#ifdef ACTIVE_TEMPORARY
    if (PDU_ptr->a.initReq->preferredMessageSize > MAXMESSAGESIZE)
    {
        display_error("BBOX", ERR_INVALID_INIT_MESS_SIZE_REQUEST, -1);
        return (FAILED);
    }
#endif

    return (SUCCESS);
}

/*****************************************************************************
 *
 * Function: tp_process_init_request
 *
 * Purpose: This function performs the following functions:
 *              - checks the init request PDU,
 *              - performs the security checks,
 *              - performs the resource verification, and
 *              - creates and sends the init response APDU to IR kernel.
 *
 * Input Arguments:
 *              PDU - init request PDU pointer
 *              data_ptr - temporary data pointer for ASN functions
 *              socketFd - the socket Fd for communicating to IR kernel
 *
 * Return values: SUCCESS or FAILED
 *
 * Output Arguments:
 *
 * External Variables:
 *
 * Function used:
 *              tp_check_init_request()
 *              FreeDeleteResultSetReq()
 *              tp_access_control()
 *              tp_resource_control()
 *              tp_create_init_response()
 *              tpa_send_APDU()
 *              display_error()
 *
 * Algorithm:
 *
 * Version: Date:   93/05/12
 *          Name:   Benjamin Yen
 *          Reason: First Draft of IR Target Kernel
 *
 * Reference:
 *
 * Notes:
 *
 ****************************************************************************/
tp_process_init_request(PDU, data_ptr, socketFd)
    APDU *PDU;
    char **data_ptr;
    int socketFd;
{
    int result;
    long APDU_length;
    SBuf APDU_ptr;
    char protocol_version, options;
    int check_password_already;

#ifdef DEBUG_LEVEL1
    fprintf(DEBUG_FILE1, "BBOX: calling tp_process_init_request\n");
#endif

    result = tp_check_init_request(PDU, &protocol_version, &options, &check_password_already);
#ifdef DEBUG_LEVEL1
    fprintf(DEBUG_FILE1, "BBOX: tp_process_init_request: check_password_already=%d\n", check_password_already);
#endif
    FreeInitializeReq(PDU->a.initReq);

    if (result == SUCCESS)
    {
        if (!check_password_already)
            result = tp_access_control(socketFd, data_ptr, INIT_REFERENCE_ID_LENGTH, INIT_REFERENCE_ID);
        else
            result = SUCCESS;        /* password has been verified */

        if (result != SUCCESS)
        {
            display_error("BBOX", ERR_ACCESS_CONTROL_FAILED, -1);
            tp_create_init_response(FALSE, &APDU_ptr, protocol_version, options, data_ptr, &APDU_length);
        }
        else
        {
            if (tp_resource_control(socketFd, data_ptr, INIT_REFERENCE_ID_LENGTH, INIT_REFERENCE_ID) == SUCCESS)
            {
                /* accepted init request */
                tp_create_init_response(TRUE, &APDU_ptr, protocol_version, options, data_ptr, &APDU_length);
            }
            else
            {
                display_error("BBOX", ERR_RESOURCE_CONTROL_FAILED, -1);
                tp_create_init_response(FALSE, &APDU_ptr, protocol_version, options, data_ptr, &APDU_length);
            }
        }
    }
    else
    {
        /* rejected init request */
        tp_create_init_response(FALSE, &APDU_ptr, protocol_version, options, data_ptr, &APDU_length);
    }

    if (tpa_send_APDU(socketFd, &APDU_ptr, APDU_length) <= 0)
    {
        display_error("TP_BBOX", ERR_SEND_DATA_TO_IR_FROM_TP, errno);
        return (FAILED);
    }

    return (SUCCESS);
}
